<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Outgoing
 */
class Hg_Command_Outgoing extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Internal command
	 *
	 * @var string
	 */
	protected $command = 'outgoing';

	/**
	 * Required options
	 *
	 * @var array
	 */
	protected $requiredOptions = array(
		'noninteractive' => null,
		'repository'     => null
	);

	/**
	 * Allowed options
	 *
	 * @var array
	 */
	protected $allowedOptions = array(
		'template' => null
	);

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct($params = null, Hg_Service $service) {
		$this->service = $service;
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Outgoing> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null,
			'repository'     => $this->service->repository->getPath()
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status > 1) {
			throw new Hg_Exception($this->getOutput('external'));
		}

		return $this;
	}

	/**
	 * Set template option
	 *
	 * @param <string> $template Template to set
	 *
	 * @return <Hg_Command_Outgoing> Instance of self
	 */
	public function template($template) {
		if ($template) {
			$this->addOption('template', '"' . $template . '\n"');
		}

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$external = new Hg_Output($this->output);
				$external->format($format);

				$output = $external;

				break;

			case 'internal':
				foreach ($this->output as $line) {
					static $internal  = array();
					static $changeset = null;

					if (preg_match('#^changeset:\s+(.*?)$#i', $line, $match)) {
						$changeset = $match[1];
					}

					if ($changeset) {
						if (preg_match('#^tag:\s+(.+?)$#i', $line, $match)) {
							$internal[$changeset]['tag'] = $match[1];
						}

						if (preg_match('#^parent:\s+(.+?)$#i', $line, $match)) {
							$internal[$changeset]['parent'] = $match[1];
						}

						if (preg_match('#^user:\s+(.+?)$#i', $line, $match)) {
							$internal[$changeset]['user'] = $match[1];
						}

						if (preg_match('#^date:\s+(.+?)$#i', $line, $match)) {
							$internal[$changeset]['date'] = $match[1];
						}

						if (preg_match('#^summary:\s+(.+?)$#i', $line, $match)) {
							$internal[$changeset]['summary'] = $match[1];
						}
					}
				}

				$output = $internal;

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}
}
