<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Executable
 */
class Hg_Executable {
	protected static $instance = null;

	protected $binary     = 'hg';
	protected $executable = null;
	protected $path       = null;

	private function __construct($path) {
		$this->setExecutable($path);
	}

	public static function getInstance($path = null) {
		if (self::$instance === null) {
			self::$instance = new Hg_Executable($path);
		}

		return self::$instance;
	}

	public function setExecutable($path) {
		if (null !== $path) {
			if (is_executable($path . DIRECTORY_SEPARATOR . $this->binary)) {
				$executables[] = $path . DIRECTORY_SEPARATOR . $this->binary;
				$this->setPath($path);
			}
		}

		if ((empty($executables)) || (null === $path)) {
			$paths = explode(PATH_SEPARATOR, $_SERVER['PATH']);

			foreach ($paths as $path) {
				if (is_executable($path . DIRECTORY_SEPARATOR . $this->binary)) {
					$executables[] = $path . DIRECTORY_SEPARATOR . $this->binary;
					$this->setPath($path);
				}
			}

			if (count($executables) === 0) {
				throw new Hg_Exception('Mercurial could not be found on this system');
			}
		}

		$this->executable = array_shift($executables);

		return $this;
	}

	public function setPath($path) {
		$this->path = $path;

		return $this;
	}

	public function getExecutable() {
		return $this->executable;
	}

	public function getPath() {
		return $this->path;
	}

	public function __toString() {
		return $this->getExecutable();
	}
}