<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Service
 */
class Hg_Service {
	protected $repository;
	protected $executable;
	protected $output;

	public function __construct($repository) {
		$this->executable = Hg_Executable::getInstance();
		$this->repository = new Hg_Repository($this, $repository);
	}

	public function setRepository($repository) {
		$this->repository = new Hg_Repository($repository);
	}

	public function setExecutable($executable) {
		$this->executable = Hg_Executable::getInstance();
	}

	public function __call($method, $arguments) {
		$command = new Hg_Proxy($this);
		return call_user_func_array(array($command, $method), $arguments);
	}

	public function __get($property) {
		switch ($property) {
			case 'executable':
				return $this->executable;

				break;

			case 'repository':
				return $this->repository;

				break;

			case 'output':
				return $this->output;

				break;

			default:
				throw new Hg_Exception(sprintf('The property (%s) you are trying to get is not implemented yet.', $property));

				break;
		}
	}

	public function __set($property, $value) {
		switch ($property) {
			case 'repository':
				$this->repository = new Hg_Repository($value);

				break;

			case 'output':
				$this->output = new Hg_Output($value);

				break;

			default:
				throw new Hg_Exception(sprintf('The property (%s) you are trying to set is not implemented yet.', $property));

				break;
		}
	}
}
